#!/usr/bin/ksh

#
#	ksh implementation of stanza
#

# . /funcrc

STZ_ERR_SYNTAX=1
STZ_ERR_MSG_SYNTAX="Stanza API syntax error"
STZ_ERR_FILE=2
STZ_ERR_MSG_FILE="Stanza file not found"
STZ_ERR_READONLY=3
STZ_ERR_MSG_READONLY="Stanza file is read-only"
STZ_ERR_LOCKED=4
STZ_ERR_MSG_LOCKED="Stanza lock file exists"
STZ_ERR_FORMAT=5
STZ_ERR_MSG_FORMAT="Bad stanza file format"
STZ_ERR_SECTION=6
STZ_ERR_MSG_SECTION="Stanza section not found"
STZ_ERR_VARIABLE=7
STZ_ERR_MSG_VARIABLE="Stanza variable not found"

function stanza_getallsects
{
[ -n "$debug" ] && set -x
	typeset line

	while read line; do
		if [[ "$line" = +([!=]): ]]; then
			echo ${line%:}
		fi
	done

	return 0
}

function stanza_getsomesects
{
[ -n "$debug" ] && set -x
	typeset varname=$1
	typeset value=$2
	typeset section found line

	if [[ "$value" != \"*\" ]]; then
		value="\"$value\""
	fi

	while read line; do
		if [[ "$line" = +([!=]): ]]; then
			break
		fi
	done

	while true; do
		section=${line%:}
		found=0
		while read line; do
		    if [[ "$line" = +([!=]): ]]; then
			found=1
			break
		    else
			if [[ "${line%%=*}" = "$varname" ]]; then
			    if [[ "${line#*=}" = "$value" ]]; then
				echo $section
			    fi
			fi
		    fi
		done
		if [[ $found = 0 ]]; then
			return 0
		fi
	done
}

function stanza_getdata
{
[ -n "$debug" ] && set -x
	typeset section=$1
	typeset line

	while read line; do
		if [[ "$line" = +([!=]): ]]; then
			if [[ "${line%:}" = "$section" ]]; then
			    while read line; do
				if [[ "$line" = +([!=]): ]]; then
					return 0;
				elif [[ "$line" != "" ]]; then
					echo $line
				fi
			    done
			    return 0
			fi
		fi
	done

	echo $STZ_ERR_MSG_SECTION >&2
	return $STZ_ERR_SECTION
}

function stanza_getvars
{
[ -n "$debug" ] && set -x
	typeset section=$1
	typeset line

	while read line; do
		if [[ "$line" = +([!=]): ]]; then
			if [[ "${line%:}" = "$section" ]]; then
			    while read line; do
				if [[ "$line" = +([!=]): ]]; then
					return 0;
				else
					echo ${line%%=*}
				fi
			    done
			    return 0;
			fi
		fi
	done

	echo $STZ_ERR_MSG_SECTION >&2
	return $STZ_ERR_SECTION
}

function stanza_get
{
[ -n "$debug" ] && set -x
	typeset section=$1
	typeset varname=$2
	typeset line

	while read line; do
		if [[ "$line" = +([!=]): ]]; then
			if [[ "${line%:}" = "$section" ]]; then
			    while read line; do
				if [[ "$line" = +([!=]): ]]; then
				    echo $STZ_ERR__MSG_VARIABLE >&2
				    return $STZ_ERR_VARIABLE
				else
				    if [[ "${line%%=*}" = "$varname" ]]; then
					result=${line#*=}
					if [[ "$result" != \"*\" ]]; then
						echo "$result"
					else
						result="${result#\"}"
						echo "${result%\"}"
					fi
					return 0
				    fi
				fi
			    done
			    echo $STZ_ERR__MSG_VARIABLE >&2
			    return $STZ_ERR_VARIABLE
			fi
		fi
	done

	echo $STZ_ERR_MSG_SECTION >&2
	return $STZ_ERR_SECTION
}

function stanza_set
{
[ -n "$debug" ] && set -x
	typeset section=$1
	typeset varname=$2
	typeset value=$3
	typeset found line

	if [[ "$value" != \"*\" ]]; then
		value="\"$value\""
	fi

	found=0
	while read line; do
		if [[ "$line" = *([ 	])\#*\|"" ]]; then
			echo $line
			continue
		elif [[ "$line" = +([!=]): ]]; then
			echo $line
			if [[ "${line%:}" = "$section" ]]; then
				found=1
				break
			fi
		else
			echo "	$line"
		fi
	done
	if [[ $found = 0 ]]; then
		echo $STZ_ERR_MSG_SECTION >&2
		return $STZ_ERR_SECTION
	fi

	found=0
	while read line; do
		if [[ "$line" = \#*\|"" ]]; then
			echo $line
			continue
		elif [[ "$line" = +([!=]): ]]; then
			if [[ $found = 0 ]]; then
				echo "	$varname=$value"
			fi
			echo $line
			break
		elif [[ "${line%%=*}" = "$varname" ]]; then
			found=1
			echo "	$varname=$value"
		else
			echo "	$line"
		fi
	done

	while read line; do
		if [[ "$line" = \#*\|"" ]]; then
			echo $line
		elif [[ "$line" = +([!=]): ]]; then
			echo $line
		else
			echo "	$line"
		fi
	done

	return 0
}

function stanza_error
{
	case $1 in
	$STZ_ERR_SYNTAX)
		echo "$STZ_ERR_MSG_SYNTAX"
		;;
	$STZ_ERR_FILE)
		echo "$STZ_ERR_MSG_FILE"
		;;
	$STZ_ERR_READONLY)
		echo "$STZ_ERR_MSG_READONLY"
		;;
	$STZ_ERR_LOCKED)
		echo "$STZ_ERR_MSG_LOCKED"
		;;
	$STZ_ERR_FORMAT)
		echo "$STZ_ERR_MSG_FORMAT"
		;;
	$STZ_ERR_SECTION)
		echo "$STZ_ERR_MSG_SECTION"
		;;
	$STZ_ERR_VARIABLE)
		echo "$STZ_ERR_MSG_VARIABLE"
		;;
	esac

	return 0
}

function trim
{
	typeset line

	while read line; do
		if [[ "$line" != \#*\|"" ]]; then
			echo $line
		fi
	done
}

function sh_mv
{
	sh_ln $1 $2 && sh_rm $1
	return $?
}

function stanza
{
	if [ "$1" != "errorstring" ]; then
		if [ ! -r $2 ]; then
			echo $STZ_ERR_MSG_FILE >&2
			return $STZ_ERR_FILE
		fi
	fi

	case $1 in
	getsects)
		case $# in
		2)
			trim < $2 | stanza_getallsects
			return $?
			;;
		4)
			trim < $2 | stanza_getsomesects $3 $4
			return $?
			;;
		*)
			echo $STZ_ERR_MSG_SYNTAX >&2
			return $STZ_ERR_SYNTAX
			;;
		esac
		;;
	getdata)
		if [ $# != 3 ]; then
			echo $STZ_ERR_MSG_SYNTAX >&2
			return $STZ_ERR_SYNTAX
		fi
		trim < $2 | stanza_getdata $3
		return $?
		;;
	getvars)
		if [ $# != 3 ]; then
			echo $STZ_ERR_MSG_SYNTAX >&2
			return $STZ_ERR_SYNTAX
		fi
		trim < $2 | stanza_getvars $3
		return $?
		;;
	get)
		if [ $# != 4 ]; then
			echo $STZ_ERR_MSG_SYNTAX >&2
			return $STZ_ERR_SYNTAX
		fi
		trim < $2 | stanza_get $3 $4
		return $?
		;;
	set)
		if [ $# != 5 ]; then
			echo $STZ_ERR_MSG_SYNTAX >&2
			return $STZ_ERR_SYNTAX
		fi
		if [ ! -w $2 ]; then
			echo $STZ_ERR_MSG_READONLY >&2
			return $STZ_ERR_READONLY
		fi
		sh_cp $2 $2.tmp >/dev/null 2>&1
		if sh_mv $2.tmp $2.lck >/dev/null 2>&1; then :
		else
			sh_rm $2.tmp >/dev/null 2>&1
			echo $STZ_ERR_MSG_LOCKED >&2
			return $STZ_ERR_LOCKED
		fi
		stanza_set $3 $4 $5 < $2.lck > $2
		sh_rm $2.lck
		return $?
		;;
	errorstring)
		if [ $# != 2 ]; then
			echo $STZ_ERR_MSG_SYNTAX >&2
			return $STZ_ERR_SYNTAX
		fi
		stanza_error $2
		return $?
		;;
	*)
		echo $STZ_ERR_MSG_SYNTAX >&2
		return $STZ_ERR_SYNTAX
		;;
	esac
}
